import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        domWebGL.appendChild(renderer.domElement);
        //renderer.outputEncoding = THREE.sRGBEncoding;
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0x050505);
        this._scene = scene;

        this._setupCamera();
        this._setupLight();
        this._setupModel();
        this._setupGUI();

        this._setupControls();
        this._setupHelper();
        
        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupModel() {
        const geometry = new THREE.BufferGeometry();
        
        const indices = [];
        const vertices = [];
        const normals = [];
        const colors = [];

        const size = 20;
        const segments = 10;

        const halfSize = size / 2;
        const segmentSize = size / segments;

        for(let i=0; i<=segments; i++) {
            const y = (i*segmentSize) - halfSize;
            for(let j=0; j<=segments; j++) {
                const x = (j*segmentSize) - halfSize;

                vertices.push(x, -y, 0);
                normals.push(0, 0, 1);
                colors.push((x/size)+0.5, (y/size)+0.5, 1);
            }
        }

        for(let i=0; i<segments; i++) {
            for(let j=0; j<segments; j++) {
                const a = i * (segments+1) + (j+1);
                const b = i * (segments+1) + j;
                const c = (i+1) * (segments+1) + j;
                const d = (i+1) * (segments+1) + (j+1);

                indices.push(a, b, d);
                indices.push(b, c, d);
            }
        }

        geometry.setIndex(indices);
        geometry.setAttribute("position", new THREE.Float32BufferAttribute(vertices, 3));
        geometry.setAttribute("normal", new THREE.Float32BufferAttribute(normals, 3));
        geometry.setAttribute("color", new THREE.Float32BufferAttribute(colors, 3));

        const material = new THREE.MeshPhongMaterial( {
            side: THREE.DoubleSide,
            vertexColors: true
        });

        const mesh = new THREE.Mesh(geometry, material);
        this._scene.add(mesh);

        this._material = material;
        this._mesh = mesh;
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);

        const gui = new GUI();
        gui.add(this._material, "wireframe");

        this._stats = stats;
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(27, window.innerWidth / window.innerHeight, 1, 3500);
        camera.position.z = 64;

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        const light = new THREE.HemisphereLight();
        this._scene.add(light);
    }

    update() {
        const time = Date.now() * 0.001;
        
        this._mesh.rotation.x = time * 0.25;
        this._mesh.rotation.y = time * 0.5;

        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

        this._renderer.setPixelRatio( window.devicePixelRatio );
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    (new App()).render();
}